from __future__ import annotations

import json
import os
from random import random
from typing import TYPE_CHECKING, Any
from unittest.mock import patch

import pandas as pd
import pytest
from pymatgen.core import Lattice, Structure
from pytest import CaptureFixture

from matbench_discovery import FIGSHARE_DIR, ROOT, Key
from matbench_discovery.data import (
    DATA_FILES,
    as_dict_handler,
    df_wbm,
    figshare_versions,
    glob_to_df,
    load,
)

if TYPE_CHECKING:
    from pathlib import Path


with open(f"{FIGSHARE_DIR}/{figshare_versions[-1]}.json") as file:
    figshare_urls = json.load(file)["files"]

structure = Structure(
    lattice=Lattice.cubic(5),
    species=("Fe", "O"),
    coords=((0, 0, 0), (0.5, 0.5, 0.5)),
)


@pytest.mark.parametrize(
    "key, hydrate",
    [
        ("wbm_summary", True),
        ("wbm_initial_structures", True),
        ("wbm_computed_structure_entries", False),
        ("mp_elemental_ref_entries", True),
        ("mp_energies", True),
    ],
)
def test_load(
    df_float: pd.DataFrame,
    # df with Structures and ComputedStructureEntries as dicts
    df_with_pmg_objects: pd.DataFrame,
    capsys: CaptureFixture[str],
    tmp_path: Path,
    key: str,
    hydrate: bool,
) -> None:
    filepath = DATA_FILES[key]
    # intercept HTTP requests and write dummy df to disk instead
    with patch("urllib.request.urlretrieve") as url_retrieve:
        writer = df_with_pmg_objects.to_json if ".json" in filepath else df_float.to_csv
        url_retrieve.side_effect = lambda _url, path: writer(path)
        out = load(
            key,
            hydrate=hydrate,
            # test both str and Path for cache_dir
            cache_dir=str(tmp_path) if random() < 0.5 else tmp_path,
        )

    stdout, _stderr = capsys.readouterr()

    assert f"Downloading {key!r} from {figshare_urls[key][0]}" in stdout

    # check we called read_csv/read_json once for each data_name
    assert url_retrieve.call_count == 1

    assert isinstance(out, pd.DataFrame), f"{key} not a DataFrame"

    # test that df loaded from cache is the same as initial df
    from_cache = load(key, hydrate=hydrate, cache_dir=tmp_path)
    pd.testing.assert_frame_equal(out, from_cache)


def test_load_raises(tmp_path: Path) -> None:
    key = "bad-key"
    with pytest.raises(ValueError) as exc:  # noqa: PT011
        load(key)

    assert f"Unknown {key=}, must be one of {list(DATA_FILES)}" in str(exc.value)

    version = "invalid-version"
    with pytest.raises(ValueError) as exc:  # noqa: PT011
        load("wbm_summary", version=version, cache_dir=tmp_path)

    assert (
        str(exc.value) == f"Unexpected {version=}. Must be one of {figshare_versions}."
    )
    assert os.listdir(tmp_path) == [], "cache_dir should be empty"


def test_load_doc_str() -> None:
    doc_str = load.__doc__
    assert isinstance(doc_str, str)  # mypy type narrowing

    # check that we link to the right data description page
    with open(f"{ROOT}/site/package.json") as file:
        pkg = json.load(file)  # get repo URL from package.json
    route = "/contribute"
    assert f"{pkg['homepage']}/contribute" in doc_str
    assert os.path.isdir(f"{ROOT}/site/src/routes/{route}")


wbm_summary_expected_cols = {
    "uncorrected_energy_from_cse",
    Key.bandgap_pbe,
    Key.dft_energy,
    Key.e_form_raw,
    Key.e_form_wbm,
    Key.e_form,
    Key.each_wbm,
    Key.formula,
    Key.n_sites,
    Key.volume,
    Key.wyckoff,
}


# TODO skip this test if offline
# @pytest.mark.skipif(online, reason="requires internet connection")
@pytest.mark.parametrize(
    "file_key, version, expected_shape, expected_cols",
    [
        ("mp_elemental_ref_entries", figshare_versions[-1], (9, 89), set()),
        pytest.param(
            "wbm_summary",
            figshare_versions[-1],
            (256_963, 17),
            wbm_summary_expected_cols,
            marks=pytest.mark.slow,  # run pytest -m 'slow' to select this marker
        ),
        pytest.param(
            # large file but needed to test loading compressed JSON from URL
            "mp_computed_structure_entries",
            figshare_versions[-1],
            (154_718, 1),
            {"entry"},
            marks=pytest.mark.very_slow,
        ),
    ],
)
def test_load_no_mock(
    file_key: str,
    version: str,
    expected_shape: tuple[int, int],
    expected_cols: set[str],
    capsys: CaptureFixture[str],
    tmp_path: Path,
) -> None:
    assert os.listdir(tmp_path) == [], "cache_dir should be empty"
    # This function runs the download from Figshare for real hence takes some time and
    # requires being online
    df = load(file_key, version=version, cache_dir=tmp_path)
    assert len(os.listdir(tmp_path)) == 1, "cache_dir should have one file"
    assert df.shape == expected_shape
    assert (
        set(df) >= expected_cols
    ), f"Loaded df missing columns {expected_cols - set(df)}"

    stdout, stderr = capsys.readouterr()
    assert stderr == ""
    rel_path = getattr(type(DATA_FILES), file_key)
    cache_path = f"{tmp_path}/{rel_path}"
    assert (
        f"Downloading {file_key!r} from {figshare_urls[file_key][0]}\nCached "
        f"{file_key!r} to {cache_path!r}" in stdout
    )

    # test that df loaded from cache is the same as initial df
    pd.testing.assert_frame_equal(
        df, load(file_key, version=version, cache_dir=tmp_path)
    )

    stdout, stderr = capsys.readouterr()
    assert stderr == ""
    assert stdout == f"Loading {file_key!r} from cached file at {cache_path!r}\n"


def test_as_dict_handler() -> None:
    class C:
        def as_dict(self) -> dict[str, Any]:
            return {"foo": "bar"}

    assert as_dict_handler(C()) == {"foo": "bar"}
    assert as_dict_handler(1) is None
    assert as_dict_handler("foo") is None
    assert as_dict_handler([1, 2, 3]) is None
    assert as_dict_handler({"foo": "bar"}) is None


def test_df_wbm() -> None:
    assert df_wbm.shape == (256_963, 18)
    assert df_wbm.index.name == Key.mat_id
    assert set(df_wbm) > {Key.formula, Key.mat_id, Key.bandgap_pbe}


@pytest.mark.parametrize("pattern", ["*df.csv", "*df.json"])
def test_glob_to_df(pattern: str, tmp_path: Path, df_mixed: pd.DataFrame) -> None:
    os.makedirs(f"{tmp_path}", exist_ok=True)
    df_mixed.to_csv(f"{tmp_path}/dummy_df.csv", index=False)
    df_mixed.to_json(f"{tmp_path}/dummy_df.json")

    df_out = glob_to_df(f"{tmp_path}/{pattern}")
    assert df_out.shape == df_mixed.shape
    assert list(df_out) == list(df_mixed)

    with pytest.raises(FileNotFoundError):
        glob_to_df("foo")



from __future__ import annotations

import gzip
import json
import os
import pickle
import sys
import urllib.error
import urllib.request
from glob import glob
from typing import TYPE_CHECKING, Any, Callable

import pandas as pd
from monty.json import MontyDecoder
from tqdm import tqdm

from matbench_discovery import FIGSHARE_DIR, Key

if TYPE_CHECKING:
    from pathlib import Path

    from pymatgen.analysis.phase_diagram import PatchedPhaseDiagram

# ruff: noqa: T201

# repo URL to raw files on GitHub
RAW_REPO_URL = "https://github.com/janosh/matbench-discovery/raw"
figshare_versions = sorted(
    x.split(os.path.sep)[-1].split(".json")[0] for x in glob(f"{FIGSHARE_DIR}/*.json")
)
# directory to cache downloaded data files
default_cache_dir = os.getenv(
    "MATBENCH_DISCOVERY_CACHE_DIR",
    f"{os.path.expanduser('~/.cache/matbench-discovery')}/{figshare_versions[-1]}",
)


def as_dict_handler(obj: Any) -> dict[str, Any] | None:
    """Pass this to json.dump(default=) or as pandas.to_json(default_handler=) to
    serialize Python classes with as_dict(). Warning: Objects without a as_dict() method
    are replaced with None in the serialized data.
    """
    try:
        return obj.as_dict()  # all MSONable objects implement as_dict()
    except AttributeError:
        return None  # replace unhandled objects with None in serialized data
        # removes e.g. non-serializable AseAtoms from M3GNet relaxation trajectories


def load(
    key: str,
    version: str = figshare_versions[-1],
    cache_dir: str | Path = default_cache_dir,
    hydrate: bool = False,
    **kwargs: Any,
) -> pd.DataFrame | PatchedPhaseDiagram:
    """Download parts of or the full MP training data and WBM test data as pandas
    DataFrames. The full training and test sets are each about ~500 MB as compressed
    JSON which will be cached locally to cache_dir for faster re-loading unless
    cache_dir is set to None.

    See matbench_discovery.data.DATA_FILES for recognized data keys. See [here]
    (https://janosh.github.io/matbench-discovery/contribute#--direct-download) for file
    descriptions.

    Args:
        key (str): Which parts of the MP/WBM data to load. Must be one of
            list(DATA_FILES).
        version (str, optional): Which version of the dataset to load. Defaults to
            latest version of data files published to Figshare. Pass any invalid version
            to see valid options.
        cache_dir (str, optional): Where to cache data files on local drive. Defaults to
            '~/.cache/matbench-discovery'. Set to None to disable caching.
        hydrate (bool, optional): Whether to hydrate pymatgen objects. If False,
            Structures and ComputedStructureEntries are returned as dictionaries which
            can be hydrated on-demand with df.col.map(Structure.from_dict). Defaults to
            False as it noticeably increases load time.
        **kwargs: Additional keyword arguments passed to pandas.read_json or read_csv,
            depending on which file is loaded.

    Raises:
        ValueError: On bad version number or bad data_key.

    Returns:
        pd.DataFrame: Single dataframe or dictionary of dfs if multiple data requested.
    """
    if version not in figshare_versions:
        raise ValueError(f"Unexpected {version=}. Must be one of {figshare_versions}.")

    if not isinstance(key, str) or key not in DATA_FILES:
        raise ValueError(f"Unknown {key=}, must be one of {list(DATA_FILES)}.")

    with open(f"{FIGSHARE_DIR}/{version}.json") as json_file:
        file_urls = json.load(json_file)["files"]

    file_path = DataFiles.__dict__[key]

    cache_path = f"{cache_dir}/{file_path}"
    if not os.path.isfile(cache_path):  # download from Figshare URL
        url = file_urls[key][0]
        print(f"Downloading {key!r} from {url}")
        try:
            # ensure directory exists
            os.makedirs(os.path.dirname(cache_path), exist_ok=True)
            # download and save to disk
            urllib.request.urlretrieve(url, cache_path)
            print(f"Cached {key!r} to {cache_path!r}")
        except urllib.error.HTTPError as exc:
            raise ValueError(f"Bad {url=}") from exc
        except Exception:
            print(f"\n\nvariable dump:\n{file_path=},\n{url=}")
            raise

    print(f"Loading {key!r} from cached file at {cache_path!r}")
    if ".pkl" in file_path:  # handle key='mp_patched_phase_diagram' separately
        with gzip.open(cache_path, "rb") as zip_file:
            return pickle.load(zip_file)

    csv_ext = (".csv", ".csv.gz", ".csv.bz2")
    reader = pd.read_csv if file_path.endswith(csv_ext) else pd.read_json
    try:
        df = reader(cache_path, **kwargs)
    except Exception:
        print(f"\n\nvariable dump:\n{file_path=},\n{reader=}\n{kwargs=}")
        raise

    if Key.mat_id in df:
        df = df.set_index(Key.mat_id)
    if hydrate:
        for col in df:
            if not isinstance(df[col].iloc[0], dict):
                continue
            try:
                # convert dicts to pymatgen Structures and ComputedStructureEntries
                df[col] = [
                    MontyDecoder().process_decoded(dct)
                    for dct in tqdm(df[col], desc=col)
                ]
            except Exception:
                print(f"\n\nvariable dump:\n{col=},\n{df[col]=}")
                raise

    return df


def glob_to_df(
    pattern: str,
    reader: Callable[[Any], pd.DataFrame] | None = None,
    pbar: bool = True,
    **kwargs: Any,
) -> pd.DataFrame:
    """Combine data files matching a glob pattern into a single dataframe.

    Args:
        pattern (str): Glob file pattern.
        reader (Callable[[Any], pd.DataFrame], optional): Function that loads data from
            disk. Defaults to pd.read_csv if ".csv" in pattern else pd.read_json.
        pbar (bool, optional): Whether to show progress bar. Defaults to True.
        **kwargs: Keyword arguments passed to reader (i.e. pd.read_csv or pd.read_json).

    Returns:
        pd.DataFrame: Combined dataframe.
    """
    reader = reader or pd.read_csv if ".csv" in pattern.lower() else pd.read_json

    # prefix pattern with ROOT if not absolute path
    files = glob(pattern)
    if len(files) == 0:
        raise FileNotFoundError(f"No files matching glob {pattern=}")

    sub_dfs = {}  # used to join slurm job array results into single df
    for file in tqdm(files, disable=not pbar):
        df = reader(file, **kwargs)
        sub_dfs[file] = df

    return pd.concat(sub_dfs.values())


class Files(dict):  # type: ignore[type-arg]
    """Files instance inherits from dict so that .values(), items(), etc. are supported
    but also allows accessing attributes by dot notation. E.g. FILES.some_file_key.
    This enables tab completion in IDEs and auto-updating attribute names across the
    code base when changing the key of a file.
    """

    def __init__(
        self, root: str = default_cache_dir, key_map: dict[str, str] | None = None
    ) -> None:
        """Create a Files instance.

        Args:
            root (str, optional): Root directory used to absolufy every file path.
            Defaults to '~/.cache/matbench-discovery/[latest_figshare_release]' where
                latest_figshare_release is e.g. 1.0.0. Can also be set through env var
                MATBENCH_DISCOVERY_CACHE_DIR.
            key_map (dict[str, str], optional): Mapping from attribute names to keys in
                the dict. Useful if you want to have keys like 'foo+bar' that are not
                valid Python identifiers. Defaults to None.
        """
        # callback that's triggered when a file corresponding to a class attribute does
        # not exist. gets passed key and missing filepath. set to None to disable
        self._on_not_found: Callable[[str, str], None] | None = lambda key, path: print(
            f"Warning: {path!r} associated with {key=} does not exist.",
            file=sys.stderr,
        )

        rel_paths = {
            (key_map or {}).get(key, key): file
            for key, file in type(self).__dict__.items()
            if not key.startswith("_")
        }
        abs_paths = {key: f"{root}/{file}" for key, file in rel_paths.items()}
        self.__dict__ = abs_paths
        super().__init__(abs_paths)

    def __getattribute__(self, key: str) -> str:
        """Override __getattr__ to check if key matches a file attribute."""
        filepath = super().__getattribute__(key)
        if key in self and not os.path.isfile(filepath) and self._on_not_found:
            self._on_not_found(key, filepath)
        return filepath


class DataFiles(Files):
    """Data files provided by Matbench Discovery.
    See https://janosh.github.io/matbench-discovery/contribute for data descriptions.
    """

    def _on_not_found(self, key: str, path: str) -> None:  # type: ignore[override]
        msg = (
            f"{path!r} associated with {key=} does not exist. Would you like to "
            "download it now? This will cache the file for future use."
        )

        # default to 'y' if not in interactive session, and user can't answer
        answer = "" if sys.stdin.isatty() else "y"
        while answer not in ("y", "n"):
            answer = input(f"{msg} [y/n] ").lower().strip()
        if answer == "y":
            load(key)  # download and cache data file

    # TODO maybe set attrs to None and load file names from Figshare json
    mp_computed_structure_entries = (
        "mp/2023-02-07-mp-computed-structure-entries.json.gz"
    )
    mp_elemental_ref_entries = "mp/2023-02-07-mp-elemental-reference-entries.json.gz"
    mp_energies = "mp/2023-01-10-mp-energies.csv.gz"
    mp_patched_phase_diagram = "mp/2023-02-07-ppd-mp.pkl.gz"
    mp_trj_extxyz = "mp/2023-11-22-mp-trj-extxyz-by-yuan.zip"
    # snapshot of every task (calculation) in MP as of 2023-03-16 (14 GB)
    all_mp_tasks = "mp/2023-03-16-all-mp-tasks.zip"

    wbm_computed_structure_entries = (
        "wbm/2022-10-19-wbm-computed-structure-entries.json.bz2"
    )
    wbm_initial_structures = "wbm/2022-10-19-wbm-init-structs.json.bz2"
    wbm_cses_plus_init_structs = (
        "wbm/2022-10-19-wbm-computed-structure-entries+init-structs.json.bz2"
    )
    wbm_summary = "wbm/2023-12-13-wbm-summary.csv.gz"

    alignn_checkpoint = "2023-06-02-pbenner-best-alignn-model.pth.zip"


# data files can be downloaded and cached with matbench_discovery.data.load()
DATA_FILES = DataFiles()


df_wbm = load("wbm_summary")
df_wbm[Key.mat_id] = df_wbm.index



from glob import glob
from typing import Any

import yaml

from matbench_discovery import ROOT

MODEL_DIRS = glob(f"{ROOT}/models/*/")
MODEL_METADATA: dict[str, dict[str, Any]] = {}

for model_dir in MODEL_DIRS:
    # skip directories without python files
    if glob(f"{model_dir}*.py") == []:
        continue
    md_files = glob(f"{model_dir}metadata*.yml")
    if len(md_files) != 1:
        raise RuntimeError(f"expected 1 metadata file, got {md_files=} in {model_dir=}")
    md_file = md_files[0]
    if md_file.endswith("aborted.yml"):
        continue
    # make sure all required keys are non-empty
    with open(md_file) as yml_file:
        models = yaml.full_load(yml_file)

    # some metadata files contain a single model, some have multiple
    if not isinstance(models, list):
        models = [models]
    for model in models:
        model["model_dir"] = model_dir
        MODEL_METADATA[model["model_name"]] = model



"""Global variables used all across the matbench_discovery package."""

import json
import os
import warnings
from datetime import datetime
from enum import StrEnum, unique
from importlib.metadata import Distribution

import matplotlib.pyplot as plt
import plotly.express as px
import plotly.io as pio
from pymatviz.utils import styled_html_tag

pkg_name = "matbench-discovery"
direct_url = Distribution.from_name(pkg_name).read_text("direct_url.json") or ""
pkg_is_editable = json.loads(direct_url).get("dir_info", {}).get("editable", False)

PKG_DIR = os.path.dirname(__file__)
# repo root directory if editable install, else the pkg directory
ROOT = os.path.dirname(PKG_DIR) if pkg_is_editable else PKG_DIR
DATA_DIR = f"{ROOT}/data"  # directory to store raw data
SITE_FIGS = f"{ROOT}/site/src/figs"  # directory for interactive figures
# directory to write model analysis for website
SITE_MODELS = f"{ROOT}/site/src/routes/models"
SCRIPTS = f"{ROOT}/scripts"  # model and date analysis scripts
PDF_FIGS = f"{ROOT}/paper/figs"  # directory for light-themed PDF figures

for directory in (SITE_FIGS, SITE_MODELS, PDF_FIGS):
    os.makedirs(directory, exist_ok=True)

os.makedirs(MP_DIR := f"{DATA_DIR}/mp", exist_ok=True)
os.makedirs(WBM_DIR := f"{DATA_DIR}/wbm", exist_ok=True)
# JSON files with URLS to data files on figshare
os.makedirs(FIGSHARE_DIR := f"{ROOT}/data/figshare", exist_ok=True)

# directory to store model checkpoints downloaded from wandb cloud storage
CHECKPOINT_DIR = f"{ROOT}/wandb/checkpoints"
# wandb <entity>/<project name> to record new runs to
WANDB_PATH = "janosh/matbench-discovery"

# threshold on hull distance for a material to be considered stable
STABILITY_THRESHOLD = 0

timestamp = f"{datetime.now():%Y-%m-%d@%H-%M-%S}"
today = timestamp.split("@")[0]

# filter pymatgen warnings that spam the logs when e.g. applying corrections to
# ComputedStructureEntries or using PatchedPhaseDiagram to get e_above_hull
# warnings are:
# > No electronegativity for Ne. Setting to NaN. This has no physical meaning
# and MaterialsProject2020Compatibility to get formation energies
# > Failed to guess oxidation states for Entry
warnings.filterwarnings(action="ignore", category=UserWarning, module="pymatgen")


@unique
class Key(StrEnum):
    """Keys used to access dataframes columns."""

    arity = "arity"
    bandgap_pbe = "bandgap_pbe"
    chem_sys = "chemical_system"
    composition = "composition"
    cse = "computed_structure_entry"
    dft_energy = "uncorrected_energy"
    e_form = "e_form_per_atom_mp2020_corrected"
    e_form_pred = "e_form_per_atom_pred"
    e_form_raw = "e_form_per_atom_uncorrected"
    e_form_wbm = "e_form_per_atom_wbm"
    each = "energy_above_hull"  # as returned by MP API
    each_pred = "e_above_hull_pred"
    each_true = "e_above_hull_mp2020_corrected_ppd_mp"
    each_wbm = "e_above_hull_wbm"
    final_struct = "relaxed_structure"
    forces = "forces"
    form_energy = "formation_energy_per_atom"
    formula = "formula"
    init_struct = "initial_structure"
    magmoms = "magmoms"
    mat_id = "material_id"
    model_mean_each = "Mean prediction all models"
    model_mean_err = "Mean error all models"
    model_std_each = "Std. dev. over models"
    n_sites = "n_sites"
    site_nums = "site_nums"
    spacegroup = "spacegroup"
    stress = "stress"
    stress_trace = "stress_trace"
    struct = "structure"
    task_id = "task_id"
    # lowest WBM structures for a given prototype that isn't already in MP
    uniq_proto = "unique_prototype"
    volume = "volume"
    wyckoff = "wyckoff_spglib"  # relaxed structure Aflow label
    init_wyckoff = "wyckoff_spglib_initial_structure"  # initial structure Aflow label


@unique
class Task(StrEnum):
    """Thermodynamic stability prediction task types."""

    IS2RE = "IS2RE"  # initial structure to relaxed energy
    RS2RE = "RS2RE"  # relaxed structure to relaxed energy
    S2EFSM = "S2EFSM"  # structure to energy force stress magmom
    S2EFS = "S2EFS"  # structure to energy force stress
    S2RE = "S2RE"  # structure to relaxed energy (for models that learned a discrete
    # version of the PES like CGCNN+P)
    RP2RE = "RP2RE"  # relaxed prototype to relaxed energy
    IP2RE = "IP2RE"  # initial prototype to relaxed energy


# load figshare 1.0.0
with open(f"{FIGSHARE_DIR}/1.0.0.json") as file:
    FIGSHARE_URLS = json.load(file)


# --- start global plot settings

ev_per_atom = styled_html_tag(
    "(eV/atom)", tag="span", style="font-size: 0.8em; font-weight: lighter;"
)
quantity_labels = dict(
    n_atoms="Atom Count",
    n_elems="Element Count",
    crystal_sys="Crystal system",
    spg_num="Space group",
    n_wyckoff="Number of Wyckoff positions",
    n_sites="Number of atoms",
    energy_per_atom=f"Energy {ev_per_atom}",
    e_form=f"DFT E<sub>form</sub> {ev_per_atom}",
    e_above_hull=f"E<sub>hull dist</sub> {ev_per_atom}",
    e_above_hull_mp2020_corrected_ppd_mp=f"DFT E<sub>hull dist</sub> {ev_per_atom}",
    e_above_hull_pred=f"Predicted E<sub>hull dist</sub> {ev_per_atom}",
    e_above_hull_mp=f"E<sub>above MP hull</sub> {ev_per_atom}",
    e_above_hull_error=f"Error in E<sub>hull dist</sub> {ev_per_atom}",
    vol_diff="Volume difference (A^3)",
    e_form_per_atom_mp2020_corrected=f"DFT E<sub>form</sub> {ev_per_atom}",
    e_form_per_atom_pred=f"Predicted E<sub>form</sub> {ev_per_atom}",
    material_id="Material ID",
    band_gap="Band gap (eV)",
    formula="Formula",
    stress="σ (eV/Å³)",  # noqa: RUF001
    stress_trace="1/3 Tr(σ) (eV/Å³)",  # noqa: RUF001
)
model_labels = dict(
    alignn="ALIGNN",
    alignn_ff="ALIGNN FF",
    alignn_pretrained="ALIGNN Pretrained",
    bowsr_megnet="BOWSR",
    chgnet="CHGNet",
    chgnet_megnet="CHGNet→MEGNet",
    cgcnn_p="CGCNN+P",
    cgcnn="CGCNN",
    m3gnet_megnet="M3GNet→MEGNet",
    m3gnet="M3GNet",
    m3gnet_direct="M3GNet DIRECT",
    m3gnet_ms="M3GNet MS",
    mace="MACE",
    megnet="MEGNet",
    megnet_rs2re="MEGNet RS2RE",
    voronoi_rf="Voronoi RF",
    wrenformer="Wrenformer",
    pfp="PFP",
    dft="DFT",
    wbm="WBM",
)
px.defaults.labels = quantity_labels | model_labels


global_layout = dict(
    paper_bgcolor="rgba(0,0,0,0)",
    font_size=13,
    # increase legend marker size and make background transparent
    legend=dict(itemsizing="constant", bgcolor="rgba(0, 0, 0, 0)"),
)
pio.templates["global"] = dict(layout=global_layout)
pio.templates.default = "plotly_dark+global"
px.defaults.template = "plotly_dark+global"

# https://github.com/plotly/Kaleido/issues/122#issuecomment-994906924
# when seeing MathJax "loading" message in exported PDFs,
# use pio.kaleido.scope.mathjax = None


plt.rc("font", size=14)
plt.rc("legend", fontsize=16, title_fontsize=16)
plt.rc("axes", titlesize=16, labelsize=16)
plt.rc("savefig", bbox="tight", dpi=200)
plt.rc("figure", dpi=200, titlesize=16)
plt.rcParams["figure.constrained_layout.use"] = True
# --- end global plot settings
